@@ -1,6 +1,7 @@
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2010 Nick Schermer <nick@xfce.org>
+ * Copyright (c) 2010 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -21,10 +22,17 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+
 #include <gtk/gtk.h>
+
 #include <libxfce4util/libxfce4util.h>
+
 #include <libxfce4ui/libxfce4ui.h>
+
 #include <libxfce4panel/libxfce4panel.h>
+#include <libxfce4panel/xfce-panel-macros.h>
+
 #include <thunar-tpa/thunar-tpa-bindings.h>
 
 
@@ -47,6 +55,12 @@ GType           thunar_tpa_get_type            (void);
 void            thunar_tpa_register_type       (XfcePanelTypeModule *type_module);
 static void     thunar_tpa_finalize            (GObject             *object);
 static void     thunar_tpa_construct           (XfcePanelPlugin     *panel_plugin);
+
+#if defined (LIBXFCE4PANEL_CHECK_VERSION) && LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+static gboolean thunar_tpa_size_changed        (XfcePanelPlugin     *panel_plugin,
+                                                gint                 size);
+#endif
+
 static void     thunar_tpa_error               (ThunarTpa           *plugin,
                                                 GError              *error);
 static void     thunar_tpa_state               (ThunarTpa           *plugin,
@@ -139,6 +153,12 @@ thunar_tpa_class_init (ThunarTpaClass *klass)
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = thunar_tpa_construct;
+
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
+  plugin_class->size_changed = thunar_tpa_size_changed;
+#endif
+#endif
 }
 
 
@@ -224,6 +244,13 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 {
   ThunarTpa *plugin = THUNAR_TPA (panel_plugin);
 
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
+  /* make the plugin fit a single row */
+  xfce_panel_plugin_set_small (panel_plugin, TRUE);
+#endif
+#endif
+
   /* add the "Empty Trash" menu item */
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (plugin->mi));
 
@@ -233,6 +260,25 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 
 
 
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
+static gboolean
+thunar_tpa_size_changed (XfcePanelPlugin *panel_plugin,
+                         gint             size)
+{
+  g_return_val_if_fail (panel_plugin != NULL, FALSE);
+
+  /* make the plugin fit a single row */
+  size /= xfce_panel_plugin_get_nrows (panel_plugin);
+  gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
+
+  return TRUE;
+}
+#endif
+#endif
+
+
+
 static void
 thunar_tpa_error (ThunarTpa *plugin,
                   GError    *error)
@@ -395,10 +441,9 @@ thunar_tpa_drag_data_received (GtkWidget        *button,
   if (G_LIKELY (info == TARGET_TEXT_URI_LIST))
     {
       /* check if the data is valid for text/uri-list */
-      if (G_LIKELY (selection_data->length >= 0 && selection_data->format == 8))
+      uri_list = gtk_selection_data_get_uris (selection_data);
+      if (G_LIKELY (uri_list != NULL))
         {
-          /* parse the URI list according to RFC 2483 */
-          uri_list = g_uri_list_extract_uris ((const gchar *) selection_data->data);
           succeed = thunar_tpa_move_to_trash (plugin, (const gchar **) uri_list);
           g_strfreev (uri_list);
         }
@@ -460,6 +505,7 @@ static void
 thunar_tpa_display_trash (ThunarTpa *plugin)
 {
   gchar *display_name;
+  gchar *startup_id;
 
   g_return_if_fail (THUNAR_IS_TPA (plugin));
 
@@ -472,7 +518,9 @@ thunar_tpa_display_trash (ThunarTpa *plugin)
 
       /* schedule a new call */
       display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
-      plugin->display_trash_call = org_xfce_Trash_display_trash_async (plugin->proxy, display_name, thunar_tpa_display_trash_reply, plugin);
+      startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
+      plugin->display_trash_call = org_xfce_Trash_display_trash_async (plugin->proxy, display_name, startup_id, thunar_tpa_display_trash_reply, plugin);
+      g_free (startup_id);
       g_free (display_name);
     }
 }
@@ -483,6 +531,7 @@ static void
 thunar_tpa_empty_trash (ThunarTpa *plugin)
 {
   gchar *display_name;
+  gchar *startup_id;
 
   g_return_if_fail (THUNAR_IS_TPA (plugin));
 
@@ -495,7 +544,9 @@ thunar_tpa_empty_trash (ThunarTpa *plugin)
 
       /* schedule a new call */
       display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
-      plugin->empty_trash_call = org_xfce_Trash_empty_trash_async (plugin->proxy, display_name, thunar_tpa_empty_trash_reply, plugin);
+      startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
+      plugin->empty_trash_call = org_xfce_Trash_empty_trash_async (plugin->proxy, display_name, startup_id, thunar_tpa_empty_trash_reply, plugin);
+      g_free (startup_id);
       g_free (display_name);
     }
 }
@@ -507,6 +558,7 @@ thunar_tpa_move_to_trash (ThunarTpa    *plugin,
                           const gchar **uri_list)
 {
   gchar *display_name;
+  gchar *startup_id;
 
   g_return_val_if_fail (THUNAR_IS_TPA (plugin), FALSE);
   g_return_val_if_fail (uri_list != NULL, FALSE);
@@ -521,7 +573,9 @@ thunar_tpa_move_to_trash (ThunarTpa    *plugin,
 
   /* schedule a new call */
   display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
-  plugin->move_to_trash_call = org_xfce_Trash_move_to_trash_async (plugin->proxy, uri_list, display_name, thunar_tpa_move_to_trash_reply, plugin);
+  startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
+  plugin->move_to_trash_call = org_xfce_Trash_move_to_trash_async (plugin->proxy, uri_list, display_name, startup_id, thunar_tpa_move_to_trash_reply, plugin);
+  g_free (startup_id);
   g_free (display_name);
 
   return TRUE;
