@@ -1,4 +1,4 @@
-/* $Id: thunar-column-model.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -40,14 +40,12 @@ enum
 
 
 
-static void               thunar_column_model_class_init              (ThunarColumnModelClass *klass);
 static void               thunar_column_model_tree_model_init         (GtkTreeModelIface      *iface);
-static void               thunar_column_model_init                    (ThunarColumnModel      *column_model);
 static void               thunar_column_model_finalize                (GObject                *object);
 static GtkTreeModelFlags  thunar_column_model_get_flags               (GtkTreeModel           *tree_model);
 static gint               thunar_column_model_get_n_columns           (GtkTreeModel           *tree_model);
 static GType              thunar_column_model_get_column_type         (GtkTreeModel           *tree_model,  
-                                                                       gint                    index);
+                                                                       gint                    idx);
 static gboolean           thunar_column_model_get_iter                (GtkTreeModel           *tree_model,
                                                                        GtkTreeIter            *iter,
                                                                        GtkTreePath            *path);
@@ -55,7 +53,7 @@ static GtkTreePath       *thunar_column_model_get_path                (GtkTreeMo
                                                                        GtkTreeIter            *iter);
 static void               thunar_column_model_get_value               (GtkTreeModel           *tree_model,
                                                                        GtkTreeIter            *iter,
-                                                                       gint                    index,
+                                                                       gint                    idx,
                                                                        GValue                 *value);
 static gboolean           thunar_column_model_iter_next               (GtkTreeModel           *tree_model,
                                                                        GtkTreeIter            *iter);
@@ -108,7 +106,7 @@ struct _ThunarColumnModel
    * generated by another model.
    */
 #ifndef NDEBUG
-  guint              stamp;
+  gint               stamp;
 #endif
 
   ThunarPreferences *preferences;
@@ -119,45 +117,12 @@ struct _ThunarColumnModel
 
 
 
-static GObjectClass *thunar_column_model_parent_class;
-static guint         column_model_signals[LAST_SIGNAL];
+static guint column_model_signals[LAST_SIGNAL];
 
 
 
-GType
-thunar_column_model_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarColumnModelClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_column_model_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarColumnModel),
-        0,
-        (GInstanceInitFunc) thunar_column_model_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo tree_model_info =
-      {
-        (GInterfaceInitFunc) thunar_column_model_tree_model_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, I_("ThunarColumnModel"), &info, 0);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL, &tree_model_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarColumnModel, thunar_column_model, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, thunar_column_model_tree_model_init))
 
 
 
@@ -166,9 +131,6 @@ thunar_column_model_class_init (ThunarColumnModelClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_column_model_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_column_model_finalize;
 
@@ -272,9 +234,9 @@ thunar_column_model_get_n_columns (GtkTreeModel *tree_model)
 
 static GType
 thunar_column_model_get_column_type (GtkTreeModel *tree_model,  
-                                     gint          index)
+                                     gint          idx)
 {
-  switch (index)
+  switch (idx)
     {
     case THUNAR_COLUMN_MODEL_COLUMN_NAME:
       return G_TYPE_STRING;
@@ -331,14 +293,14 @@ thunar_column_model_get_path (GtkTreeModel *tree_model,
 static void
 thunar_column_model_get_value (GtkTreeModel *tree_model,
                                GtkTreeIter  *iter,
-                               gint          index,
+                               gint          idx,
                                GValue       *value)
 {
   ThunarColumnModel *column_model = THUNAR_COLUMN_MODEL (tree_model);
   ThunarColumn       column;
 
   _thunar_return_if_fail (THUNAR_IS_COLUMN_MODEL (column_model));
-  _thunar_return_if_fail (index < THUNAR_COLUMN_MODEL_N_COLUMNS);
+  _thunar_return_if_fail (idx < THUNAR_COLUMN_MODEL_N_COLUMNS);
   _thunar_return_if_fail (iter->stamp == column_model->stamp);
 
   /* determine the column from the iterator */
@@ -347,7 +309,7 @@ thunar_column_model_get_value (GtkTreeModel *tree_model,
   /* resolve the column according to the order */
   column = column_model->order[column];
 
-  switch (index)
+  switch (idx)
     {
     case THUNAR_COLUMN_MODEL_COLUMN_NAME:
       g_value_init (value, G_TYPE_STRING);
@@ -492,7 +454,7 @@ thunar_column_model_load_column_order (ThunarColumnModel *column_model)
 
       /* find the current position of the value */
       for (j = 0; j < THUNAR_N_VISIBLE_COLUMNS; ++j)
-        if (column_model->order[j] == value->value)
+        if (column_model->order[j] == (guint) value->value)
           break;
 
       /* check if valid */
@@ -895,16 +857,15 @@ thunar_column_model_get_column_name (ThunarColumnModel *column_model,
 {
   const gchar *column_name = _("Unknown");
   GEnumClass  *klass;
-  gint         n;
+  guint        n;
 
   _thunar_return_val_if_fail (THUNAR_IS_COLUMN_MODEL (column_model), NULL);
   _thunar_return_val_if_fail (column < THUNAR_N_VISIBLE_COLUMNS, NULL);
-  _thunar_return_val_if_fail (column >= 0, NULL);
 
   /* determine the column name from the ThunarColumn enum type */
   klass = g_type_class_ref (THUNAR_TYPE_COLUMN);
   for (n = 0; n < klass->n_values; ++n)
-    if (klass->values[n].value == column)
+    if (klass->values[n].value == (gint) column)
       column_name = _(klass->values[n].value_nick);
   g_type_class_unref (klass);
 
@@ -929,7 +890,6 @@ thunar_column_model_get_column_visible (ThunarColumnModel *column_model,
 {
   _thunar_return_val_if_fail (THUNAR_IS_COLUMN_MODEL (column_model), FALSE);
   _thunar_return_val_if_fail (column < THUNAR_N_VISIBLE_COLUMNS, FALSE);
-  _thunar_return_val_if_fail (column >= 0, FALSE);
   return column_model->visible[column];
 }
 
@@ -955,7 +915,6 @@ thunar_column_model_set_column_visible (ThunarColumnModel *column_model,
 
   _thunar_return_if_fail (THUNAR_IS_COLUMN_MODEL (column_model));
   _thunar_return_if_fail (column < THUNAR_N_VISIBLE_COLUMNS);
-  _thunar_return_if_fail (column >= 0);
 
   /* cannot change the visibility of the name column */
   if (G_UNLIKELY (column == THUNAR_COLUMN_NAME))
@@ -1010,7 +969,6 @@ thunar_column_model_get_column_width (ThunarColumnModel *column_model,
 {
   _thunar_return_val_if_fail (THUNAR_IS_COLUMN_MODEL (column_model), -1);
   _thunar_return_val_if_fail (column < THUNAR_N_VISIBLE_COLUMNS, -1);
-  _thunar_return_val_if_fail (column >= 0, -1);
   return column_model->width[column];
 }
 
@@ -1032,7 +990,6 @@ thunar_column_model_set_column_width (ThunarColumnModel *column_model,
 {
   _thunar_return_if_fail (THUNAR_IS_COLUMN_MODEL (column_model));
   _thunar_return_if_fail (column < THUNAR_N_VISIBLE_COLUMNS);
-  _thunar_return_if_fail (column >= 0);
   _thunar_return_if_fail (width >= 0);
 
   /* check if we have a new width */
