@@ -36,11 +36,6 @@ typedef struct _ThunarHistory      ThunarHistory;
 
 GType           thunar_history_get_type         (void) G_GNUC_CONST;
 
-ThunarHistory  *thunar_history_new              (void) G_GNUC_MALLOC;
-
-GtkActionGroup *thunar_history_get_action_group (const ThunarHistory *history);
-void            thunar_history_set_action_group (ThunarHistory       *history,
-                                                 GtkActionGroup      *action_group);
 ThunarFile     *thunar_history_peek_back        (ThunarHistory       *history);
 ThunarFile     *thunar_history_peek_forward     (ThunarHistory       *history);
 
