@@ -114,9 +114,10 @@ static gboolean           thunar_shortcuts_model_drag_data_get      (GtkTreeDrag
                                                                      GtkSelectionData          *selection_data);
 static gboolean           thunar_shortcuts_model_drag_data_delete   (GtkTreeDragSource         *source,
                                                                      GtkTreePath               *path);
+static void               thunar_shortcuts_model_header_visibility  (ThunarShortcutsModel      *model);
+static void               thunar_shortcuts_model_shortcut_devices   (ThunarShortcutsModel      *model);
 static void               thunar_shortcuts_model_shortcut_places    (ThunarShortcutsModel      *model);
 static void               thunar_shortcuts_model_shortcut_network   (ThunarShortcutsModel      *model);
-static gboolean           thunar_shortcuts_model_devices_load       (gpointer                   data);
 static gboolean           thunar_shortcuts_model_get_hidden         (ThunarShortcutsModel      *model,
                                                                      ThunarShortcut            *shortcut);
 static void               thunar_shortcuts_model_add_shortcut       (ThunarShortcutsModel      *model,
@@ -172,7 +173,6 @@ struct _ThunarShortcutsModel
   gchar               **hidden_bookmarks;
 
   ThunarDeviceMonitor  *device_monitor;
-  guint                 devices_monitor_idle_id;
 
   gint64                bookmarks_time;
   GFile                *bookmarks_file;
@@ -272,7 +272,7 @@ thunar_shortcuts_model_init (ThunarShortcutsModel *model)
                    G_OBJECT (model), "hidden-bookmarks");
 
   /* load volumes */
-  model->devices_monitor_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_devices_load, model, NULL);
+  thunar_shortcuts_model_shortcut_devices (model);
 
   /* add network */
   thunar_shortcuts_model_shortcut_network (model);
@@ -298,10 +298,6 @@ thunar_shortcuts_model_finalize (GObject *object)
   if (model->bookmarks_idle_id != 0)
     g_source_remove (model->bookmarks_idle_id);
 
-  /* stop device monitor loading */
-  if (model->devices_monitor_idle_id != 0)
-    g_source_remove (model->devices_monitor_idle_id);
-
   /* free all shortcuts */
   g_list_foreach (model->shortcuts, (GFunc) thunar_shortcut_free, model);
   g_list_free (model->shortcuts);
@@ -393,6 +389,9 @@ thunar_shortcuts_model_set_property (GObject      *object,
               gtk_tree_path_free (path);
             }
         }
+
+      /* update header visibility */
+      thunar_shortcuts_model_header_visibility (model);
       break;
 
     default:
@@ -425,10 +424,10 @@ thunar_shortcuts_model_get_column_type (GtkTreeModel *tree_model,
 {
   switch (idx)
     {
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER:
       return G_TYPE_BOOLEAN;
 
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM:
       return G_TYPE_BOOLEAN;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE:
@@ -528,6 +527,9 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
   gboolean        can_eject;
   GFile          *file;
   gchar          *disk_usage;
+  guint32         trash_items;
+  gchar          *trash_string;
+  gchar          *parse_name;
 
   _thunar_return_if_fail (iter->stamp == THUNAR_SHORTCUTS_MODEL (tree_model)->stamp);
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (tree_model));
@@ -539,22 +541,19 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
 
   switch (column)
     {
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value, (shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) != 0);
       break;
 
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value, (shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) == 0);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE:
       g_value_init (value, G_TYPE_BOOLEAN);
-      if (shortcut->device == NULL)
-        g_value_set_boolean (value, !shortcut->hidden);
-      else
-        g_value_set_boolean (value, !thunar_device_get_hidden (shortcut->device));
+      g_value_set_boolean (value, !shortcut->hidden);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_NAME:
@@ -590,6 +589,22 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
             }
           break;
         }
+      else if ((shortcut->group & THUNAR_SHORTCUT_GROUP_PLACES_TRASH) != 0)
+        {
+          trash_items = thunar_file_get_item_count (shortcut->file);
+          if (trash_items == 0)
+            {
+              g_value_set_static_string (value, _("Trash is empty"));
+            }
+          else
+            {
+              trash_string = g_strdup_printf (ngettext ("Trash contains %d file",
+                                                        "Trash contains %d files",
+                                                        trash_items), trash_items);
+              g_value_take_string (value, trash_string);
+            }
+          break;
+        }
       else if (shortcut->tooltip == NULL)
         {
           if (shortcut->file != NULL)
@@ -600,7 +615,11 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
             file = NULL;
 
           if (G_LIKELY (file != NULL))
-            shortcut->tooltip = g_file_get_parse_name (file);
+            {
+              parse_name = g_file_get_parse_name (file);
+              shortcut->tooltip = g_markup_escape_text (parse_name, -1);
+              g_free (parse_name);
+            }
         }
 
       g_value_set_static_string (value, shortcut->tooltip);
@@ -632,8 +651,7 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
     case THUNAR_SHORTCUTS_MODEL_COLUMN_MUTABLE:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value,
-                           shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                           || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+                           shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_CAN_EJECT:
@@ -799,15 +817,72 @@ thunar_shortcuts_model_drag_data_delete (GtkTreeDragSource *source,
 
 
 
-static gboolean
-thunar_shortcuts_model_devices_load (gpointer data)
+static void
+thunar_shortcuts_model_header_visibility (ThunarShortcutsModel *model)
 {
-  ThunarShortcutsModel *model = THUNAR_SHORTCUTS_MODEL (data);
-  ThunarShortcut       *shortcut;
-  GList                *devices;
-  GList                *lp;
+  GList          *lp;
+  ThunarShortcut *shortcut;
+  ThunarShortcut *header;
+  guint           n_children = 0;
+  GList          *lp_header = NULL;
+  gint            idx_header = 0;
+  gint            i;
+  GtkTreePath    *path;
+  GtkTreeIter     iter;
 
-  GDK_THREADS_ENTER ();
+  _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
+
+  for (lp = model->shortcuts, i = 0; lp != NULL; lp = lp->next, i++)
+    {
+      shortcut = lp->data;
+
+      if ((shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
+        {
+          if (lp_header != NULL)
+            {
+              update_header:
+
+              header = lp_header->data;
+              if (header->hidden != (n_children == 0))
+                {
+                  /* set new visibility */
+                  header->hidden = (n_children == 0);
+
+                  /* notify view */
+                  path = gtk_tree_path_new_from_indices (idx_header, -1);
+                  GTK_TREE_ITER_INIT (iter, model->stamp, lp_header);
+                  gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+                  gtk_tree_path_free (path);
+                }
+
+              if (lp == NULL)
+                return;
+            }
+
+          /* reset for new */
+          idx_header = i;
+          lp_header = lp;
+          n_children = 0;
+        }
+      else if (!shortcut->hidden)
+        {
+          n_children++;
+        }
+    }
+
+  _thunar_assert (lp == NULL);
+  if (lp_header != NULL)
+    goto update_header;
+}
+
+
+
+static void
+thunar_shortcuts_model_shortcut_devices (ThunarShortcutsModel *model)
+{
+  ThunarShortcut *shortcut;
+  GList          *devices;
+  GList          *lp;
 
   /* add the devices heading */
   shortcut = g_slice_new0 (ThunarShortcut);
@@ -831,21 +906,17 @@ thunar_shortcuts_model_devices_load (gpointer data)
   devices = thunar_device_monitor_get_devices (model->device_monitor);
   for (lp = devices; lp != NULL; lp = lp->next)
     {
-      thunar_shortcuts_model_device_added (model->device_monitor, lp->data, model);
+      thunar_shortcuts_model_device_added (NULL, lp->data, model);
       g_object_unref (G_OBJECT (lp->data));
     }
   g_list_free (devices);
 
-  GDK_THREADS_LEAVE ();
-
   /* monitor for changes */
   g_signal_connect (model->device_monitor, "device-added", G_CALLBACK (thunar_shortcuts_model_device_added), model);
   g_signal_connect (model->device_monitor, "device-removed", G_CALLBACK (thunar_shortcuts_model_device_removed), model);
   g_signal_connect (model->device_monitor, "device-changed", G_CALLBACK (thunar_shortcuts_model_device_changed), model);
 
-  model->devices_monitor_idle_id = 0;
-
-  return FALSE;
+  thunar_shortcuts_model_header_visibility (model);
 }
 
 
@@ -1094,8 +1165,7 @@ thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
       gtk_tree_path_free (path);
 
       /* check if we need to save */
-      needs_save = shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                   || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      needs_save = (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
       /* actually free the shortcut */
       thunar_shortcut_free (shortcut, model);
@@ -1103,11 +1173,30 @@ thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
       /* the shortcuts list was changed, so write the gtk bookmarks file */
       if (needs_save)
         thunar_shortcuts_model_save (model);
+
+      /* update header visibility */
+      thunar_shortcuts_model_header_visibility (model);
     }
 }
 
 
 
+static gboolean
+thunar_shortcuts_model_local_file (GFile *gfile)
+{
+  _thunar_return_val_if_fail (G_IS_FILE (gfile), FALSE);
+
+  /* schemes we'd like to have as ThunarFiles in the model */
+  if (g_file_has_uri_scheme (gfile, "file")
+      || g_file_has_uri_scheme (gfile, "computer")
+      || g_file_has_uri_scheme (gfile, "recent"))
+    return TRUE;
+
+  return FALSE;
+}
+
+
+
 static void
 thunar_shortcuts_model_load_line (GFile       *file_path,
                                   const gchar *name,
@@ -1123,7 +1212,7 @@ thunar_shortcuts_model_load_line (GFile       *file_path,
   _thunar_return_if_fail (name == NULL || g_utf8_validate (name, -1, NULL));
 
   /* handle local and remove files differently */
-  if (g_file_has_uri_scheme (file_path, "file"))
+  if (thunar_shortcuts_model_local_file (file_path))
     {
       /* try to open the file corresponding to the uri */
       file = thunar_file_get (file_path, NULL);
@@ -1153,10 +1242,11 @@ thunar_shortcuts_model_load_line (GFile       *file_path,
     {
       /* create the shortcut entry */
       shortcut = g_slice_new0 (ThunarShortcut);
-      shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
       shortcut->gicon = g_themed_icon_new ("folder-remote");
       shortcut->location = g_object_ref (file_path);
       shortcut->sort_id = row_num;
+      shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
       shortcut->name = g_strdup (name);
 
       /* append the shortcut to the list */
@@ -1180,6 +1270,9 @@ thunar_shortcuts_model_load (gpointer data)
                               thunar_shortcuts_model_load_line,
                               model);
 
+  /* update the visibility */
+  thunar_shortcuts_model_header_visibility (model);
+
   GDK_THREADS_LEAVE ();
 
   model->bookmarks_idle_id = 0;
@@ -1279,8 +1372,7 @@ thunar_shortcuts_model_save (ThunarShortcutsModel *model)
   for (lp = model->shortcuts; lp != NULL; lp = lp->next)
     {
       shortcut = THUNAR_SHORTCUT (lp->data);
-      if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-          || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+      if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
         {
           if (shortcut->file != NULL)
             uri = thunar_file_dup_uri (shortcut->file);
@@ -1388,8 +1480,8 @@ thunar_shortcuts_model_device_added (ThunarDeviceMonitor  *device_monitor,
 {
   ThunarShortcut *shortcut;
 
-  _thunar_return_if_fail (THUNAR_DEVICE_MONITOR (device_monitor));
-  _thunar_return_if_fail (model->device_monitor == device_monitor);
+  _thunar_return_if_fail (device_monitor == NULL || THUNAR_DEVICE_MONITOR (device_monitor));
+  _thunar_return_if_fail (device_monitor == NULL || model->device_monitor == device_monitor);
   _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
 
@@ -1415,6 +1507,11 @@ thunar_shortcuts_model_device_added (ThunarDeviceMonitor  *device_monitor,
 
   /* insert in the model */
   thunar_shortcuts_model_add_shortcut (model, shortcut);
+
+  /* header visibility if call is from monitor */
+  if (device_monitor != NULL
+      && !shortcut->hidden)
+    thunar_shortcuts_model_header_visibility (model);
 }
 
 
@@ -1457,6 +1554,7 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
   gint            idx;
   GtkTreePath    *path;
   ThunarShortcut *shortcut;
+  gboolean        update_header = FALSE;
 
   _thunar_return_if_fail (THUNAR_DEVICE_MONITOR (device_monitor));
   _thunar_return_if_fail (model->device_monitor == device_monitor);
@@ -1480,6 +1578,13 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
       g_free (shortcut->tooltip);
       shortcut->tooltip = NULL;
 
+      /* hidden state */
+      if (shortcut->hidden != thunar_device_get_hidden (device))
+        {
+          shortcut->hidden = thunar_device_get_hidden (device);
+          update_header = TRUE;
+        }
+
       /* generate an iterator for the path */
       GTK_TREE_ITER_INIT (iter, model->stamp, lp);
 
@@ -1488,6 +1593,10 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
       gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
       gtk_tree_path_free (path);
     }
+
+  /* header visibility */
+  if (update_header)
+    thunar_shortcuts_model_header_visibility (model);
 }
 
 
@@ -1631,8 +1740,7 @@ thunar_shortcuts_model_has_bookmark (ThunarShortcutsModel *model,
       shortcut = lp->data;
 
       /* only check bookmarks */
-      if (shortcut->group != THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-          && shortcut->group != THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+      if (shortcut->group != THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
         continue;
 
       if (shortcut->file != NULL
@@ -1694,7 +1802,8 @@ thunar_shortcuts_model_iter_for_file (ThunarShortcutsModel *model,
         }
 
       /* but maybe we have a mounted(!) volume with a matching mount point */
-      if (shortcut->device != NULL)
+      if (shortcut->device != NULL
+          && !thunar_device_get_hidden (shortcut->device))
         {
           mount_point = thunar_device_get_root (shortcut->device);
           if (mount_point != NULL)
@@ -1788,22 +1897,16 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
 
   /* create the new shortcut that will be inserted */
   shortcut = g_slice_new0 (ThunarShortcut);
+  shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
 
-  if (THUNAR_IS_FILE (file) && thunar_file_is_local (file))
-    shortcut->file = g_object_ref (G_OBJECT (file));
-  else
-    shortcut->location = g_object_ref (G_OBJECT (location));
-
-  if (g_file_has_uri_scheme (location, "file"))
+  if (thunar_shortcuts_model_local_file (location))
     {
-      shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
+      shortcut->file = thunar_file_get (location, NULL);
     }
   else
     {
-      shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      shortcut->location = g_object_ref (G_OBJECT (location));
       shortcut->gicon = g_themed_icon_new ("folder-remote");
-
-      dst_path = NULL;
     }
 
   /* add the shortcut to the list at the given position */
@@ -1934,8 +2037,7 @@ thunar_shortcuts_model_remove (ThunarShortcutsModel *model,
   shortcut = g_list_nth_data (model->shortcuts, gtk_tree_path_get_indices (path)[0]);
 
   /* verify that the shortcut is removable */
-  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                  || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
   /* remove the shortcut (using the file destroy handler) */
   thunar_shortcuts_model_remove_shortcut (model, shortcut);
@@ -1973,8 +2075,7 @@ thunar_shortcuts_model_rename (ThunarShortcutsModel *model,
   shortcut = THUNAR_SHORTCUT (((GList *) iter->user_data)->data);
 
   /* verify the shortcut */
-  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                  || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
   /* perform the rename */
   if (G_UNLIKELY (shortcut->name != NULL))
@@ -2107,4 +2208,7 @@ thunar_shortcuts_model_set_hidden (ThunarShortcutsModel *model,
   /* store new list */
   g_object_set (G_OBJECT (model->preferences), "hidden-bookmarks", bookmarks, NULL);
   g_strfreev (bookmarks);
+
+  /* header visibility */
+  thunar_shortcuts_model_header_visibility (model);
 }
